/* _kb.h -- include file used for building 'libkb'
 * Copyright (C) 1995, 1996 Markus F.X.J. Oberhumer
 * For conditions of distribution and use, see copyright notice in kb.h 
 */

/* WARNING: this file should *not* be used by applications. It is
   part of the implementation of the keyboard library and is
   subject to change. Applications should only use kb.h.
 */

#ifndef __MFX_KB_INTERNAL_H
#define __MFX_KB_INTERNAL_H

#ifdef __cplusplus
extern "C" {
#endif


/***********************************************************************
// virtual keys and shift-states for internal use
// note: these must have the highest scancodes of all keys
************************************************************************/

#define KB_SCAN_VIRTUAL                 120   /* lowest virtual key */
#define KB_SCAN_PAUSE_VIRTUAL           121
#define KB_SCAN_CONTROL_BREAK_VIRTUAL   122
#define KB_SCAN_LSHIFT_VIRTUAL          123
#define KB_SCAN_RSHIFT_VIRTUAL          124
#define KB_SCAN_LCONTROL_VIRTUAL        125

#define KB_SHIFT_VIRTUAL        0x8000


/***********************************************************************
// internal tables
************************************************************************/

extern unsigned char _kb_prefix_scancode[128];
extern unsigned char _kb_inverse_prefix_scancode[128];

extern unsigned short _kb_shift_state_table[128];
extern unsigned short _kb_unshift_state_table[128];

extern char _kb_ascii_table[128];
extern char _kb_shift_table[128];
extern unsigned char _kb_alt_table[128];
extern unsigned char _kb_control_table[128];

void _kb_init_tables(void);


/***********************************************************************
// locking and debugging
************************************************************************/

int _kb_lock(void);
void _kb_unlock(void);

void _kb_port_debug(void);
void _kb_error_debug(void);
int _kb_error_debug_avail(void);


/***********************************************************************
// BIOS keyboard access
************************************************************************/

/* cmd values for kb_bioskey() */
#define KB_KEYBRD_READ              0       /* read key */
#define KB_NKEYBRD_READ             0x10    /* read key - enhanced */
#define KB_KEYBRD_READY             1       /* check key ready */
#define KB_NKEYBRD_READY            0x11    /* check key ready - enhanced */
#define KB_KEYBRD_SHIFTSTATUS       2       /* get shift status */
#define KB_NKEYBRD_SHIFTSTATUS      0x12    /* get shift status - enhanced */

unsigned kb_bioskey(unsigned _cmd);


/***********************************************************************
// low level includes
************************************************************************/

#if (defined(__BORLANDC__) || defined(__TURBOC__)) && defined(__MSDOS__)
#  include <dos.h>
#  include <bios.h>
#  define KB_DISABLE()		disable()
#  define KB_ENABLE()		enable()
#  define KB_INP8(a)		inport(a)
#  define KB_OUTP8(a,b)		outport(a,b)
#elif defined(__EMX__)
#  include <dos.h>
#  include <sys/hw.h>
#  if 1
#    define KB_DISABLE()	/* not allowed under emx */
#    define KB_ENABLE()		/* not allowed under emx */
#  else
#    define KB_DISABLE()	__asm__("cli \n")
#    define KB_ENABLE()		__asm__("sti \n" "cld \n")
#  endif
#  define KB_INP8(a)		_inp8(a)
#  define KB_OUTP8(a,b)		_outp8(a,b)
#  define KB_INT86(a,b,c)	_int86(a,b,c)
#elif defined(__GO32__)
#  include <dos.h>
#  include <bios.h>
#  include <dpmi.h>
#  include <go32.h>
#  include <pc.h>
#  define KB_DISABLE()		disable()
#  define KB_ENABLE()		enable()
#  define KB_INP8(a)		inportb(a)
#  define KB_OUTP8(a,b)		outportb(a,b)
#  define KB_INT86(a,b,c)	int86(a,b,c)
#elif defined(_MSC_VER) && defined(MSDOS)
#  include <dos.h>
#  include <bios.h>
#  define KB_DISABLE()		disable()
#  define KB_ENABLE()		enable()
#  define KB_INP8(a)		inport(a)
#  define KB_OUTP8(a,b)		outport(a,b)
#elif defined(__WATCOMC__) && defined(MSDOS)
#  include <dos.h>
#  include <bios.h>
#  define KB_DISABLE()		_disable()
#  define KB_ENABLE()		_enable()
#  define KB_INP8(a)		inp(a)
#  define KB_OUTP8(a,b)		outp(a,b)
#else
#  error unsupported compiler
#endif


#ifdef __cplusplus
} /* extern "C" */
#endif

#endif /* already included */

/*
vi:ts=4
*/

